/*
 * Copyright 2022 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.example.healthconnect.codelab.presentation.navigation

import android.content.Intent
import androidx.compose.foundation.Image
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.material.MaterialTheme
import androidx.compose.material.ScaffoldState
import androidx.compose.material.Text
import androidx.compose.material.rememberScaffoldState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import androidx.navigation.compose.currentBackStackEntryAsState
import androidx.navigation.compose.rememberNavController
import com.example.healthconnect.codelab.R
import com.example.healthconnect.codelab.presentation.theme.HealthConnectTheme
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.launch

const val HEALTH_CONNECT_SETTINGS_ACTION = "androidx.health.ACTION_HEALTH_CONNECT_SETTINGS"

/**
 * The side navigation drawer used to explore each Health Connect feature.
 */
@Composable
fun Drawer(
    scope: CoroutineScope,
    scaffoldState: ScaffoldState,
    navController: NavController,
) {
  val navBackStackEntry by navController.currentBackStackEntryAsState()
  val currentRoute = navBackStackEntry?.destination?.route
  val activity = LocalContext.current
  Column {
    Row(
      modifier = Modifier.fillMaxWidth(),
      horizontalArrangement = Arrangement.Center
    ) {
      Image(
        modifier = Modifier
            .width(96.dp)
            .clickable {
                navController.navigate(Screen.WelcomeScreen.route) {
                    navController.graph.startDestinationRoute?.let { route ->
                        popUpTo(route) {
                            saveState = true
                        }
                    }
                    launchSingleTop = true
                    restoreState = true
                }
                scope.launch {
                    scaffoldState.drawerState.close()
                }
            },
        painter = painterResource(id = R.drawable.ic_health_connect_logo),
        contentDescription = stringResource(id = R.string.health_connect_logo)
      )
    }
    Spacer(modifier = Modifier.height(16.dp))
    Text(
      modifier = Modifier.fillMaxWidth(),
      textAlign = TextAlign.Center,
      text = stringResource(id = R.string.app_name)
    )
    Spacer(modifier = Modifier.height(16.dp))
    Screen.values().filter { it.hasMenuItem }.forEach { item ->
      DrawerItem(
        item = item,
        selected = item.route == currentRoute,
        onItemClick = {
          navController.navigate(item.route) {
            // See: https://developer.android.com/jetpack/compose/navigation#nav-to-composable
            navController.graph.startDestinationRoute?.let { route ->
              popUpTo(route) {
                saveState = true
              }
            }
            launchSingleTop = true
            restoreState = true
          }
          scope.launch {
            scaffoldState.drawerState.close()
          }
        }
      )
    }
    Spacer(modifier = Modifier.height(16.dp))
    Row(
      modifier = Modifier
          .fillMaxWidth()
          .clickable(
              onClick = {
                  val settingsIntent = Intent()
                  settingsIntent.action = HEALTH_CONNECT_SETTINGS_ACTION
                  activity.startActivity(settingsIntent)
              }
          )
          .height(48.dp)
          .padding(start = 16.dp),
      verticalAlignment = Alignment.CenterVertically
    ) {
      Text(
        text = stringResource(R.string.settings),
        style = MaterialTheme.typography.h5,
        color = MaterialTheme.colors.onBackground
      )
    }
  }
}

@Preview
@Composable
fun DrawerPreview() {
  val scope = rememberCoroutineScope()
  val scaffoldState = rememberScaffoldState()
  val navController = rememberNavController()
  HealthConnectTheme {
    Drawer(
      scope = scope,
      scaffoldState = scaffoldState,
      navController = navController
    )
  }
}
